# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['plumes']

package_data = \
{'': ['*']}

install_requires = \
['dynaconf>=3.1.0,<4.0.0',
 'fire>=0.3.1,<0.4.0',
 'python-box>=5.1.0,<6.0.0',
 'toml>=0.10.1,<0.11.0',
 'tqdm>=4.48.2,<5.0.0',
 'tweepy>=3.9.0,<4.0.0']

entry_points = \
{'console_scripts': ['plumes = plumes.cli:main']}

setup_kwargs = {
    'name': 'plumes',
    'version': '2.3.2',
    'description': 'Simple Twitter CLI for day-to-day social media hygiene',
    'long_description': '<br />\n<p align="center">\n  <a href="https://github.com/nnadeau/plumes">\n    <img src="https://raw.githubusercontent.com/nnadeau/plumes/master/media/feather.png" alt="Logo" width="80" height="80">\n  </a>\n\n  <h3 align="center">plumes</h3>\n\n  <p align="center">\n    Simple Twitter CLI for day-to-day social media hygiene\n    <br />\n    ·\n    <a href="https://github.com/nnadeau/plumes/issues">Report Bug</a>\n    ·\n    <a href="https://github.com/nnadeau/plumes/issues">Request Feature</a>\n  </p>\n</p>\n\n[![GitHub issues](https://img.shields.io/github/issues/nnadeau/plumes)](https://github.com/nnadeau/plumes/issues)\n[![GitHub forks](https://img.shields.io/github/forks/nnadeau/plumes)](https://github.com/nnadeau/plumes/network)\n[![GitHub stars](https://img.shields.io/github/stars/nnadeau/plumes)](https://github.com/nnadeau/plumes/stargazers)\n[![GitHub license](https://img.shields.io/github/license/nnadeau/plumes)](https://github.com/nnadeau/plumes/blob/master/LICENSE)\n\n[![PyPI Version](https://img.shields.io/pypi/v/plumes.svg)](https://pypi.python.org/pypi/plumes)\n[![PyPI License](https://img.shields.io/pypi/l/plumes.svg)](https://pypi.python.org/pypi/plumes)\n[![PyPI Wheel](https://img.shields.io/pypi/wheel/plumes.svg)](https://pypi.python.org/pypi/plumes)\n[![PyPI Format](https://img.shields.io/pypi/format/plumes.svg)](https://pypi.python.org/pypi/plumes)\n[![PyPI Pythons](https://img.shields.io/pypi/pyversions/plumes.svg)](https://pypi.python.org/pypi/plumes)\n[![PyPI Implementation](https://img.shields.io/pypi/implementation/plumes.svg)](https://pypi.python.org/pypi/plumes)\n\n[![semantic-release](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--release-e10079.svg)](https://github.com/semantic-release/semantic-release)\n[![Test](https://github.com/nnadeau/plumes/workflows/Test/badge.svg)](https://github.com/nnadeau/plumes/actions)\n[![Release](https://github.com/nnadeau/plumes/workflows/Release/badge.svg)](https://github.com/nnadeau/plumes/actions)\n[![Publish](https://github.com/nnadeau/plumes/workflows/Publish/badge.svg)](https://github.com/nnadeau/plumes/actions)\n\n[![Twitter](https://img.shields.io/twitter/url?style=social&url=https%3A%2F%2Fgithub.com%2Fnnadeau%2Fplumes)](https://twitter.com/intent/tweet?text=Wow:&url=https%3A%2F%2Fgithub.com%2Fnnadeau%2Fplumes)\n\n## Contents\n\n- [Contents](#contents)\n- [Overview](#overview)\n- [Inspiration](#inspiration)\n- [Installation](#installation)\n- [Usage](#usage)\n  - [Getting Started And Creating Your Authentication Config](#getting-started-and-creating-your-authentication-config)\n  - [Export Friends](#export-friends)\n  - [Export Followers](#export-followers)\n  - [Export Tweets](#export-tweets)\n  - [Audit Users](#audit-users)\n  - [Prune Your Tweets](#prune-your-tweets)\n- [Setting Up Authentication](#setting-up-authentication)\n  - [Get Your Twitter API Tokens](#get-your-twitter-api-tokens)\n  - [Configuring `plumes`](#configuring-plumes)\n- [Contributing](#contributing)\n- [Testing](#testing)\n\n## Overview\n\n`Plumes` is an open-source Python CLI app for day-to-day social media hygiene.\nIt was designed to provide a simple, clear, and concise interface to quickly explore and clean a personal Twitter account.\n\n## Inspiration\n\n[![panzer tweet](https://raw.githubusercontent.com/nnadeau/plumes/master/media/tweet-panzer.png)](https://twitter.com/panzer/status/943935357673861120)\n\n[![chrisalbon tweet](https://raw.githubusercontent.com/nnadeau/plumes/master/media/tweet-chrisalbon.png)](https://twitter.com/chrisalbon/status/1295408107078615041)\n\n\n## Installation\n\n```bash\n# python >=3.6.1 is required\npip install plumes\n```\n\n## Usage\n\n### Getting Started And Creating Your Authentication Config\n\n```bash\n# create your config file\nplumes init\n\n# validate your config file\nplumes check_config\n\n# print your config file (watch out for sensitive tokens!)\nplumes view_config\n```\n\n### Export Friends\n\n[Extract friends](examples/SteveMartinToGo-friends.json) ordered in which they were added:\n\n```bash\nplumes friends <flags>\n\n# e.g., get the friends of Steve Martin (see data in examples dir)\nplumes friends SteveMartinToGo --limit 100\n```\n\n**Arguments**:\n\n- `screen_name` _Optional[str], optional_ - Target user\'s screen name (i.e., Twitter handle). If none is given, authenticated user is used. Defaults to None.\n- `limit` _Optional[int], optional_ - Max number of users to fetch. Defaults to None.\n- `output` _Optional[str], optional_ - Output path for JSON file. Defaults to None.\n\n![Plumes friends gif](https://raw.githubusercontent.com/nnadeau/plumes/master/media/terminal-friends.gif)\n\n### Export Followers\n\n[Extract followers](examples/alyankovic-followers.json) ordered in which they were added:\n\n```bash\nplumes followers <flags>\n\n# e.g., get the followers of Al Yankovic (see data in examples dir)\nplumes followers alyankovic --limit 100\n```\n\n**Arguments**:\n\n- `screen_name` _Optional[str], optional_ - Target user\'s screen name (i.e., Twitter handle). If none is given, authenticated user is used. Defaults to None.\n- `limit` _Optional[int], optional_ - Max number of users to fetch. Defaults to None.\n- `output` _Optional[str], optional_ - Output path for JSON file. Defaults to None.\n\n### Export Tweets\n\n[Extract (and archive) tweets](examples/ConanOBrien-tweets.json) ordered in by most recent:\n\n```bash\nplumes tweets <flags>\n\n# e.g., get the tweets of Conan O\'Brien (see data in examples dir)\nplumes tweets ConanOBrien --limit 100\n```\n\n**Arguments**:\n\n- `screen_name` _Optional[str], optional_ - Target user\'s screen name (i.e., Twitter handle). If none is given, authenticated user is used. Defaults to None.\n- `limit` _Optional[int], optional_ - Max number of users to fetch. Defaults to None.\n- `output` _Optional[str], optional_ - Output path for JSON file. Defaults to None.\n\n![Plumes tweet gif](https://raw.githubusercontent.com/nnadeau/plumes/master/media/terminal-tweets.gif)\n\n### Audit Users\n\nAudit and review users given criteria.\nUse this to mass follow/unfollow many users.\n\n```bash\nplumes audit_users PATH <flags>\n\n# e.g., follow 100 of Al Yankovic\'s followers\nplumes followers alyankovic --limit 100\nplumes audit_users alyankovic-followers.json --befriend\n\n# e.g., prune (i.e., unfollow) current friends who have less than 100 followers AND haven\'t tweeted in the last 30 days\nplumes friends --output "friends.json"\nplumes audit_users "friends.json" --prune --min_followers 100 --days 30\n```\n\n**Arguments**:\n\n- `path` _str_ - Path to JSON file of users (e.g., output of friends())\n- `min_followers` _Optional[int], optional_ - Min number of followers. Defaults to None.\n- `max_followers` _Optional[int], optional_ - Max number of followers. Defaults to None.\n- `min_friends` _Optional[int], optional_ - Min number of friends. Defaults to None.\n- `max_friends` _Optional[int], optional_ - Max number of friends. Defaults to None.\n- `days` _Optional[int], optional_ - Days since last tweet. Defaults to None.\n- `min_tweets` _Optional[int], optional_ - Min number of tweets. Defaults to None.\n- `max_tweets` _Optional[int], optional_ - Max number of tweets. Defaults to None.\n- `min_favourites` _Optional[int], optional_ - Min number of favourites. Defaults to None.\n- `max_favourites` _Optional[int], optional_ - Max number of favourites. Defaults to None.\n- `min_ratio` _Optional[float], optional_ - Min Twitter follower-friend (TFF) ratio. Defaults to None.\n- `max_ratio` _Optional[float], optional_ - Max Twitter follower-friend (TFF) ratio. Defaults to None.\n- `prune` _bool, optional_ - Unfollow identified users. Defaults to False.\n- `befriend` _bool, optional_ - Follow identified users. Defaults to False.\n- `bool_or` _bool, optional_ - Switch to boolean OR for conditions. Defaults to False.\n\n### Prune Your Tweets\n\nAudit and review tweets given criteria.\nUse this to mass favourite or delete tweets.\n\n```bash\nplumes audit_tweets PATH <flags>\n\n# e.g., delete your tweets that are older than 60 days AND that you didn\'t self-favourite\nplumes tweets --output "tweets.json"\nplumes audit_tweets "tweets.json" --prune --days 60 --self_favorited False\n\n# e.g., export 100 of Conan O\'Brien\'s tweets and favourite those that have a maximum of 10 likes and a minimum of 50 retweets\nplumes tweets ConanOBrien --limit 100\nplumes audit_tweets ConanOBrien-tweets.json --favorite --max_likes 10 --min_retweets 50\n```\n\n**Arguments**:\n\n- `days` _Optional[int], optional_ - Days since tweeted. Defaults to None.\n- `min_likes` _Optional[int], optional_ - Min number of favourites. Defaults to None.\n- `max_likes` _Optional[int], optional_ - Max number of favourites. Defaults to None.\n- `min_retweets` _Optional[int], optional_ - Min number of retweets. Defaults to None.\n- `max_retweets` _Optional[int], optional_ - Max number of retweets. Defaults to None.\n- `min_ratio` _Optional[float], optional_ - Min Twitter like-retweet ratio. Defaults to None.\n- `max_ratio` _Optional[float], optional_ - Max Twitter like-retweet ratio. Defaults to None.\n- `self_favorited` _Optional[bool], optional_ - Check if tweet is self-liked. Defaults to None.\n- `prune` _bool, optional_ - Prune and destroy identified tweets. Defaults to False.\n- `favorite` _bool, optional_ - Like identified tweets. Defaults to False.\n- `bool_or` _bool, optional_ - Switch to boolean OR for conditions. Defaults to False.\n\n## Setting Up Authentication\n\n### Get Your Twitter API Tokens\n\n- Navigate to the [Twitter Dev Portal](https://developer.twitter.com/en/apps)\n- Click `Create an app`\n\n  ![Twitter app link](https://raw.githubusercontent.com/nnadeau/plumes/master/media/2020-08-20-09-11-05.png)\n\n- Fill out the forms\n- Navigate to `Keys and tokens`\n\n  ![Tokens link](https://raw.githubusercontent.com/nnadeau/plumes/master/media/2020-08-20-09-12-34.png)\n\n- Copy the values for `API key`, `API secret key`, `Access token`, and `Access token secret`\n\n  ![Keys and tokens](https://raw.githubusercontent.com/nnadeau/plumes/master/media/2020-08-20-09-13-24.png)\n\n### Configuring `plumes`\n\nThe API tokens can either be set as environment variables (using the `PLUMES_` prefix; e.g., `export PLUMES_CONSUMER_KEY=xxxxx`) or configuration variables in `~/.plumes.toml`:\n\n- CONSUMER_KEY = `<API key>`\n- CONSUMER_SECRET = `<API secret key>`\n- ACCESS_TOKEN = `<Access token>`\n- ACCESS_TOKEN_SECRET = `<Access token secret>`\n\n## Contributing\n\nPlease see [`CONTRIBUTING.md`](.github/CONTRIBUTING.md) and the [Code of Conduct](CODE_OF_CONDUCT.md) for how to contribute to the project\n\n## Testing\n\n- Please review the [`Makefile`](Makefile) for an overview of all available tests\n- The most important tests and `make` commands are highlighted below:\n\n```bash\n# auto-format code\nmake format\n\n# perform all static tests\nmake check\n```\n\n---\n\n<div>Icons made by <a href="https://smashicons.com/" title="Smashicons">Smashicons</a> from <a href="https://www.flaticon.com/" title="Flaticon">www.flaticon.com</a></div>\n',
    'author': 'Nicholas Nadeau',
    'author_email': 'nicholas.nadeau@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nnadeau/plumes',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
